/*---------------------------------------------------------------------------*
 |        PDFlib - A library for dynamically generating PDF documents        |
 +---------------------------------------------------------------------------+
 |        Copyright (c) 1997-2000 Thomas Merz. All rights reserved.          |
 +---------------------------------------------------------------------------+
 |    This software is NOT in the public domain.  It can be used under two   |
 |    substantially different licensing terms:                               |
 |                                                                           |
 |    The commercial license is available for a fee, and allows you to       |
 |    - ship a commercial product based on PDFlib                            |
 |    - implement commercial Web services with PDFlib                        |
 |    - distribute (free or commercial) software when the source code is     |
 |      not made available                                                   |
 |    Details can be found in the file PDFlib-license.pdf.                   |
 |                                                                           |
 |    The "Aladdin Free Public License" doesn't require any license fee,     |
 |    and allows you to                                                      |
 |    - develop and distribute PDFlib-based software for which the complete  |
 |      source code is made available                                        |
 |    - redistribute PDFlib non-commercially under certain conditions        |
 |    - redistribute PDFlib on digital media for a fee if the complete       |
 |      contents of the media are freely redistributable                     |
 |    Details can be found in the file aladdin-license.pdf.                  |
 |                                                                           |
 |    These conditions extend to ports to other programming languages.       |
 |    PDFlib is distributed with no warranty of any kind. Commercial users,  |
 |    however, will receive warranty and support statements in writing.      |
 *---------------------------------------------------------------------------*/

/* p_gif.c
 *
 * GIF processing for PDFlib
 *
 */

/* 
 * This module is basically a rewrite of David Koblas' giftoppm.c,
 * taken from the PBMPLUS package. It has been heavily reworked in
 * order to make it thread-safe and fit into PDFlib.
 */

#include <string.h>

#include "p_intern.h"

/* Obsolete function -- use PDF_open_image_file() instead */
PDFLIB_API int PDFLIB_CALL
PDF_open_GIF(PDF *p, const char *filename)
{
    if (PDF_SANITY_CHECK_FAILED(p))
	return -1;

    pdf_error(p, PDF_NonfatalError,
    	"PDF_open_GIF() is obsolete; use PDF_open_image_file instead");

    return PDF_open_image_file(p, "gif", filename, NULL, 0);
}

#define CM_RED			0
#define CM_GREEN		1
#define CM_BLUE			2

#define	MAX_LWZ_BITS		12

#define INTERLACE		0x40
#define LOCALCOLORMAP		0x80
#define BitSet(byte, bit)	(((byte) & (bit)) == (bit))

#define	ReadOK(file,buffer,len)	(fread(buffer, len, 1, file) != 0)

#define LM_to_uint(a,b)			(((b)<<8)|(a))

static int ReadColorMap(FILE *fp, int number, pdf_colormap buffer);
static int DoExtension(pdf_image *image, int label);
static int GetDataBlock(pdf_image *image, unsigned char  *buf);
static void ReadImage(PDF *p, pdf_image *image, PDF_data_source *src);

static void
pdf_data_source_GIF_init(PDF *p, PDF_data_source *src)
{
    pdf_image		*image;

    image = (pdf_image *) src->private_data;

    src->buffer_length	= (size_t) (image->width * image->height * 1);
    src->buffer_start	= (byte*) p->malloc(p, src->buffer_length,
				"pdf_data_source_GIF_init");
    src->bytes_available= 0;
    src->next_byte	= NULL;

    if (!image->info.gif.useGlobalColormap) {
	if (ReadColorMap(image->fp, image->palette_size, *image->colormap)) {
	    PDF_close_image(p, image - p->images);
	    pdf_error(p, PDF_IOError, "Error reading local colormap");
	}
    }
}

static pdf_bool
pdf_data_source_GIF_fill(PDF *p, PDF_data_source *src)
{
    pdf_image	*image;

    if (src->next_byte != NULL)		/* all finished in one turn */
	return pdf_false;

    image = (pdf_image *) src->private_data;

    src->next_byte = src->buffer_start;
    src->bytes_available = src->buffer_length;

    ReadImage(p, image, src);

    /* avoid annoying compiler warning "unreferenced parameter" */
    if (p)
	return pdf_true;

    return pdf_true;
}

static void
pdf_data_source_GIF_terminate(PDF *p, PDF_data_source *src)
{
  p->free(p, (void *) src->buffer_start);
}

int
pdf_open_GIF_data(PDF *p, int imageslot, const char *filename, const char *stringparam, int intparam)
{
    unsigned char	buf[16];
    char	c;
    int		imageCount = 0;
    char	version[4];
    int		imageNumber = 1;	/* we read only the first image */
    pdf_image	*image;
    int		mask = -1;

    image = &p->images[imageslot];

    if (stringparam && *stringparam) {
	if (!strcmp(stringparam, "mask"))
	    pdf_error(p, PDF_ValueError, "Cannot handle GIF image mask");

	else if (!strcmp(stringparam, "masked")) {
	    mask = intparam;

	    if (mask < 0)
		pdf_error(p, PDF_ValueError,
		    "Invalid image mask (no %d) for image '%s'", mask,filename);

	    if (mask >= p->images_capacity || !p->images[mask].in_use ||
		p->images[mask].colorspace != ImageMask) {
		pdf_error(p, PDF_ValueError,
		    "Bad image mask (no %d) for image '%s'", mask, filename);
	    }
	} else if (!strcmp(stringparam, "ignoremask")) {
	    /* we invert this flag later */
	    image->transparent = pdf_true;
	} else
	    pdf_error(p, PDF_ValueError,
	    	"Unknown parameter %s in pdf_open_GIF", stringparam);
    }

    image->colormap 	= (pdf_colormap*) p->malloc(p, sizeof(pdf_colormap),
    					"pdf_open_GIF");
    image->mask		= mask;

    if ((image->fp = fopen(filename, READMODE)) == NULL)
	return -1;	/* Couldn't open GIF file */

#define PDF_STRING_GIF	"\107\111\106"
#define PDF_STRING_87a	"\070\067\141"
#define PDF_STRING_89a	"\070\071\141"

    /* Error reading magic number or not a GIF file */
    if (!ReadOK(image->fp,buf,6) || 
	strncmp((const char *) buf, PDF_STRING_GIF, 3) != 0) {
	fclose(image->fp);
	return -1;
    }

    strncpy(version, (const char *) buf + 3, 3);
    version[3] = '\0';

    /* Bad version number */
    if ((strcmp(version, PDF_STRING_87a) != 0) &&
        (strcmp(version, PDF_STRING_89a) != 0)) {
	fclose(image->fp);
	return -1;
    }

    /* Failed to read screen descriptor */
    if (! ReadOK(image->fp,buf,7)) {
	fclose(image->fp);
	return -1;
    }

    /* size of the global color table*/
    image->palette_size = 2 << (buf[4] & 0x07);

    if (BitSet(buf[4], LOCALCOLORMAP)) {	/* Global Colormap */
	if (ReadColorMap(image->fp, image->palette_size, *image->colormap)) {
	    fclose(image->fp);	/* Error reading global colormap */
	    return -1;
	}
    }

    /* translate the aspect ratio to PDFlib notation */
    if (buf[6] != 0) {
	image->dpi_x = -(buf[6] + ((float) 15.0)) / ((float) 64.0);
	image->dpi_y = (float) -1.0;
    }

    for (;;) {
	/* EOF / read error in image data */
	if (!ReadOK(image->fp, &c,1)) {
	    fclose(image->fp);
	    return -1;
	}

	if (c == PDF_SEMICOLON) {		/* GIF terminator */
	    /* Not enough images found in file */
	    if (imageCount < imageNumber) {
		fclose(image->fp);
		return -1;
	    }
	    break;
	}

	if (c == PDF_EXCLAM) { 	/* Extension */
	    if (!ReadOK(image->fp, &c,1)) {
		/* EOF / read error on extension function code */
		fclose(image->fp);
		return -1;
	    }
	    DoExtension(image, (int) c);
	    continue;
	}

	if (c != PDF_COMMA) {		/* Not a valid start character */
	    /* Bogus character, ignoring */
	    continue;
	}

	++imageCount;

	if (! ReadOK(image->fp, buf, 9)) {
	    /* Couldn't read left/top/width/height */
	    fclose(image->fp);
	    return -1;
	}

	image->colorspace	= Indexed;
	image->components	= 1;
	image->bpc		= 8;		/* HACK */
	image->width		= LM_to_uint(buf[4],buf[5]);
	image->height		= LM_to_uint(buf[6],buf[7]);
	image->info.gif.useGlobalColormap = !BitSet(buf[8], LOCALCOLORMAP);

#ifdef ORIG
	image->info.gif.bitPixel = 1<<((buf[8]&0x07)+1);
#endif
	image->info.gif.interlace= BitSet(buf[8], INTERLACE);

	if (imageCount == 1)
	    break;
    }
    
    image->src.init		= pdf_data_source_GIF_init;
    image->src.fill		= pdf_data_source_GIF_fill;
    image->src.terminate	= pdf_data_source_GIF_terminate;
    image->src.private_data	= (void *) image;

    image->in_use		= pdf_true;		/* mark slot as used */
    image->filename		= pdf_strdup(p, filename);

    pdf_put_image(p, imageslot);

    fclose(image->fp);

    p->free(p, image->colormap);

    return imageslot;
}

static int
ReadColorMap(FILE *fp, int number, pdf_colormap buffer)
{
    int		i;
    unsigned char	rgb[3];

    for (i = 0; i < number; ++i) {
	if (! ReadOK(fp, rgb, sizeof(rgb))) {
	    return pdf_true;
	}

	buffer[i][0] = rgb[0] ;
	buffer[i][1] = rgb[1] ;
	buffer[i][2] = rgb[2] ;
    }
    return pdf_false;
}

static int
DoExtension(pdf_image *image, int label)
{
#define buf	image->info.gif.DoExtension_buf

    switch ((unsigned char) label) {
	case 0x01:		/* Plain Text Extension */
	    break;

	case 0xff:		/* Application Extension */
	    break;

	case 0xfe:		/* Comment Extension */
	    while (GetDataBlock(image, (unsigned char*) buf) != 0) {
		/* */
	    }
	    return pdf_false;

	case 0xf9:		/* Graphic Control Extension */
	    (void) GetDataBlock(image, (unsigned char*) buf);

	    if ((buf[0] & 0x1) != 0) {
		image->transparent = !image->transparent;
		image->transval[0] = buf[3];
	    }

	    while (GetDataBlock(image, (unsigned char*) buf) != 0)
		    ;
	    return pdf_false;

	default:
	    break;
    }

	while (GetDataBlock(image, (unsigned char*) buf) != 0)
		;

	return pdf_false;
#undef buf
}

static int
GetDataBlock(pdf_image *image, unsigned char *buf)
{
	unsigned char	count;
	FILE *fp = image->fp;

	if (!ReadOK(fp, &count, 1)) {
		return -1;	/* Error in getting DataBlock size */
	}

	image->info.gif.ZeroDataBlock = (count == 0);

	if ((count != 0) && (!ReadOK(fp, buf, count))) {
		return -1;	/* Error in reading DataBlock */
	}

	return count;
}

static int
GetCode(PDF *p, pdf_image *image, int code_size, int flag)
{
#define i_curbit	(image->info.gif.curbit)
#define i_lastbit	(image->info.gif.lastbit)
#define i_done		(image->info.gif.done)
#define i_last_byte 	(image->info.gif.last_byte)
#define i_buf 		(image->info.gif.buf)

int		i, j, ret;
unsigned char	count;

    if (flag) {
	i_curbit = 0;
	i_lastbit = 0;
	i_last_byte = 0;
	i_done = pdf_false;
	return 0;
    }

    if ( (i_curbit+code_size) >= i_lastbit) {
	if (i_done) {
	    if (i_curbit >= i_lastbit) {
		fclose(image->fp);
		pdf_error(p, PDF_IOError,
		    "Corrupt GIF file '%s'", image->filename);
	    }
	    return -1;
	}
	i_buf[0] = i_buf[i_last_byte-2];
	i_buf[1] = i_buf[i_last_byte-1];

	if ((count = (unsigned char) GetDataBlock(image, &i_buf[2])) == 0)
	    i_done = pdf_true;

	i_last_byte = 2 + count;
	i_curbit = (i_curbit - i_lastbit) + 16;
	i_lastbit = (2+count)*8 ;
    }

    ret = 0;

    for (i = i_curbit, j = 0; j < code_size; ++i, ++j)
	ret |= ((i_buf[ i / 8 ] & (1 << (i % 8))) != 0) << j;

    i_curbit += code_size;

    return ret;

#undef i_curbit
#undef i_lastbit
#undef i_done
#undef i_last_byte
#undef i_buf
}

static int
LWZReadByte(PDF *p, pdf_image *image, int flag, int input_code_size)
{
#define i_fresh		(image->info.gif.fresh)
#define i_code_size	(image->info.gif.code_size)
#define i_set_code_size	(image->info.gif.set_code_size)
#define i_max_code	(image->info.gif.max_code)
#define i_max_code_size	(image->info.gif.max_code_size)
#define i_firstcode	(image->info.gif.firstcode)
#define i_oldcode	(image->info.gif.oldcode)
#define i_clear_code	(image->info.gif.clear_code)
#define i_end_code	(image->info.gif.end_code)
#define i_sp		(image->info.gif.sp)
#define i_table		(p->gif_table)
#define i_stack		(p->gif_stack)

    int		code, incode;
    register int	i;

    if (flag) {
	i_set_code_size = input_code_size;
	i_code_size = i_set_code_size+1;
	i_clear_code = 1 << i_set_code_size ;
	i_end_code = i_clear_code + 1;
	i_max_code_size = 2*i_clear_code;
	i_max_code = i_clear_code+2;

	GetCode(p, image, 0, pdf_true);
	
	i_fresh = pdf_true;

	for (i = 0; i < i_clear_code; ++i) {
	    i_table[0][i] = 0;
	    i_table[1][i] = i;
	}
	for (; i < (1<<MAX_LWZ_BITS); ++i)
	    i_table[0][i] = i_table[1][0] = 0;

	i_sp = i_stack;

	return 0;

    } else if (i_fresh) {
	i_fresh = pdf_false;
	do {
	    i_firstcode = i_oldcode = GetCode(p, image, i_code_size, pdf_false);
	} while (i_firstcode == i_clear_code);
	return i_firstcode;
    }

    if (i_sp > i_stack)
	return *--i_sp;

    while ((code = GetCode(p, image, i_code_size, pdf_false)) >= 0) {
	if (code == i_clear_code) {
	    for (i = 0; i < i_clear_code; ++i) {
		    i_table[0][i] = 0;
		    i_table[1][i] = i;
	    }
	    for (/* */ ; i < (1<<MAX_LWZ_BITS); ++i)
		i_table[0][i] = i_table[1][i] = 0;

	    i_code_size = i_set_code_size+1;
	    i_max_code_size = 2*i_clear_code;
	    i_max_code = i_clear_code+2;
	    i_sp = i_stack;
	    i_firstcode = i_oldcode = GetCode(p, image, i_code_size, pdf_false);
	    return i_firstcode;

	} else if (code == i_end_code) {
	    unsigned char	buf[260];

	    if (image->info.gif.ZeroDataBlock)
		return -2;

	    while (GetDataBlock(image, buf) > 0)
		/* */ ;

	    return -2;
	}

	incode = code;

	if (code >= i_max_code) {
	    *i_sp++ = i_firstcode;
	    code = i_oldcode;
	}

	while (code >= i_clear_code) {
	    *i_sp++ = i_table[1][code];
	    if (code == i_table[0][code]) {
		return -1;
	    }
	    code = i_table[0][code];
	}

	*i_sp++ = i_firstcode = i_table[1][code];

	if ((code = i_max_code) < (1<<MAX_LWZ_BITS)) {
	    i_table[0][code] = i_oldcode;
	    i_table[1][code] = i_firstcode;
	    ++i_max_code;

	    if ((i_max_code >= i_max_code_size) &&
		(i_max_code_size < (1<<MAX_LWZ_BITS))) {
		i_max_code_size *= 2;
		++i_code_size;
	    }
	}

	i_oldcode = incode;

	if (i_sp > i_stack)
	    return *--i_sp;
    }
    return code;

#undef i_fresh
#undef i_code_size
#undef i_set_code_size
#undef i_max_code
#undef i_max_code_size
#undef i_firstcode
#undef i_oldcode
#undef i_clear_code
#undef i_end_code
#undef i_sp
#undef i_table
#undef i_stack
}

static void
ReadImage(PDF *p, pdf_image *image, PDF_data_source *src)
{
    unsigned char	c;	
    int		v;
    unsigned int	xpos = 0, ypos = 0, pass = 0;
    byte		*dest;

    /*
     *  Initialize the Compression routines
     */
    image->info.gif.fresh = pdf_false;
    image->info.gif.ZeroDataBlock = pdf_false;

    if (!ReadOK(image->fp, &c,1) || LWZReadByte(p, image, pdf_true, c) < 0) {
	fclose(image->fp);
	pdf_error(p, PDF_IOError, "Corrupt GIF file '%s'", image->filename);
    }

    dest = src->buffer_start;
    while ((v = LWZReadByte(p, image, pdf_false, c)) >= 0 ) {

	*dest++ = (byte) v;	/* fetch pixel value */
	++xpos;

	if (xpos == (unsigned int) image->width) {
	    xpos = 0;

	    if (image->info.gif.interlace) {
		switch (pass) {
		    case 0:
		    case 1:
			ypos += 8; break;
		    case 2:
			ypos += 4; break;
		    case 3:
			ypos += 2; break;
		}

		if (ypos >= (unsigned int) image->height) {
		    ++pass;
		    switch (pass) {
		    case 1:
			ypos = 4; break;
		    case 2:
			ypos = 2; break;
		    case 3:
			ypos = 1; break;
		    default:
			goto fini;
		    }
		}
	    } else {
		++ypos;
	    }
	    dest = src->buffer_start + ypos * image->width;
	}
	if (ypos >= (unsigned int) image->height)
	    break;
    }

fini:
    if (LWZReadByte(p, image, pdf_false, c) >= 0)
	pdf_error(p, PDF_NonfatalError,
	    "Too much input data in GIF file '%s', ignoring extra.",
	    image->filename);
}
