/*---------------------------------------------------------------------------*
 |        PDFlib - A library for dynamically generating PDF documents        |
 +---------------------------------------------------------------------------+
 |        Copyright (c) 1997-2000 Thomas Merz. All rights reserved.          |
 +---------------------------------------------------------------------------+
 |    This software is NOT in the public domain.  It can be used under two   |
 |    substantially different licensing terms:                               |
 |                                                                           |
 |    The commercial license is available for a fee, and allows you to       |
 |    - ship a commercial product based on PDFlib                            |
 |    - implement commercial Web services with PDFlib                        |
 |    - distribute (free or commercial) software when the source code is     |
 |      not made available                                                   |
 |    Details can be found in the file PDFlib-license.pdf.                   |
 |                                                                           |
 |    The "Aladdin Free Public License" doesn't require any license fee,     |
 |    and allows you to                                                      |
 |    - develop and distribute PDFlib-based software for which the complete  |
 |      source code is made available                                        |
 |    - redistribute PDFlib non-commercially under certain conditions        |
 |    - redistribute PDFlib on digital media for a fee if the complete       |
 |      contents of the media are freely redistributable                     |
 |    Details can be found in the file aladdin-license.pdf.                  |
 |                                                                           |
 |    These conditions extend to ports to other programming languages.       |
 |    PDFlib is distributed with no warranty of any kind. Commercial users,  |
 |    however, will receive warranty and support statements in writing.      |
 *---------------------------------------------------------------------------*/

/* p_tiff.c
 *
 * TIFF processing for PDFlib
 *
 */

#include <string.h>

#include "p_intern.h"

/* Obsolete function -- use PDF_open_image_file() instead */
PDFLIB_API int PDFLIB_CALL
PDF_open_TIFF(PDF *p, const char *filename)
{
    if (PDF_SANITY_CHECK_FAILED(p))
	return -1;

    pdf_error(p, PDF_NonfatalError,
    	"PDF_open_TIFF() is obsolete; use PDF_open_image_file instead");
    return PDF_open_image_file(p, "tiff", filename, NULL, 0);
}

#ifdef HAVE_LIBTIFF

static void
pdf_data_source_TIFF_init(PDF *p, PDF_data_source *src)
{
    pdf_image	*image;

    image = (pdf_image *) src->private_data;
    image->info.tiff.cur_line = 0;

    if (image->compression == ccitt) {
	/* malloc is done in the fill function */
	src->buffer_length = (size_t) 0;
	src->buffer_start = (byte *) NULL;
    } else {
	src->buffer_length = (size_t) (image->components * image->width);
	src->buffer_start = (byte *)
	    p->malloc(p, src->buffer_length, "pdf_data_source_TIFF_init");
    }
}

static pdf_bool
pdf_data_source_TIFF_fill(PDF *p, PDF_data_source *src)
{
    pdf_image	*image;
    int		col;
    byte	*dest;
    uint16	fillorder;
    uint32	*s, *bc;


    image = (pdf_image *) src->private_data;

    if (image->compression == ccitt) {
	if (image->info.tiff.cur_line ==
	    (int) TIFFNumberOfStrips(image->info.tiff.tif))
	    return pdf_false;

	TIFFGetField(image->info.tiff.tif, TIFFTAG_STRIPBYTECOUNTS, &bc);

	if (bc[image->info.tiff.cur_line] > src->buffer_length) {
	    src->buffer_length = bc[image->info.tiff.cur_line];
	    src->buffer_start = (byte *) p->realloc(p, src->buffer_start,
		src->buffer_length, "pdf_data_source_TIFF_fill");
	}

	TIFFReadRawStrip(image->info.tiff.tif,
			(tstrip_t) image->info.tiff.cur_line,
			(tdata_t) src->buffer_start,
			(tsize_t) bc[image->info.tiff.cur_line]);

	src->next_byte = src->buffer_start;
	src->bytes_available = bc[image->info.tiff.cur_line];

	if (image->compression == ccitt) {
	    if (TIFFGetField(image->info.tiff.tif, TIFFTAG_FILLORDER,&fillorder)
		&& (fillorder == FILLORDER_LSB2MSB)) {
		TIFFReverseBits((unsigned char *) src->buffer_start,
		    (unsigned long) src->bytes_available);
	    }
	}

	image->info.tiff.cur_line++;

    } else {
	if (image->info.tiff.cur_line++ == image->height)
	    return pdf_false;

	src->next_byte = src->buffer_start;
	src->bytes_available = src->buffer_length;

	dest = src->buffer_start;
	s = image->info.tiff.raster + 
	    (image->height - image->info.tiff.cur_line) * image->width;
      
	switch (image->components) {
	  case 1:
	    for (col = 0; col < image->width; col++, s++) {
		    *dest++ = (byte) TIFFGetR(*s);
	    }
	    break;

	  case 3:
	    for (col = 0; col < image->width; col++, s++) {
		*dest++ = (byte) TIFFGetR(*s);
		*dest++ = (byte) TIFFGetG(*s);
		*dest++ = (byte) TIFFGetB(*s);
	    }
	    break;

	  default:
	    pdf_error(p, PDF_IOError,
		"Unknown color space in TIFF image %s (%d components)", 
		image->filename, image->components);
	}
    }
    return pdf_true;
}

static void
pdf_data_source_TIFF_terminate(PDF *p, PDF_data_source *src)
{
    p->free(p, (void *) src->buffer_start);
}

int
pdf_open_TIFF_data(PDF *p, int imageslot, const char *filename, const char *stringparam, int intparam)
{
    int32 w, h;
    uint16 unit, bpc, compression, photometric;
    uint16 *rmap, *gmap, *bmap;
    uint32 group3opts;
    tsample_t components;
    size_t npixels;
    pdf_image *image;
    int mask = -1;
    float res_x, res_y;

    image = &p->images[imageslot];

    /* Suppress all warnings and error messages */
    (void) TIFFSetErrorHandler(NULL);
    (void) TIFFSetWarningHandler(NULL);

    image->info.tiff.tif = TIFFOpen(filename, "r");

    if (!image->info.tiff.tif)
	return -1;

    if (TIFFGetField(image->info.tiff.tif, TIFFTAG_PHOTOMETRIC, &photometric) &&
	(photometric == PHOTOMETRIC_YCBCR ||
	photometric == PHOTOMETRIC_CIELAB ||
	photometric == PHOTOMETRIC_MASK ||
	photometric == PHOTOMETRIC_SEPARATED)) {
	/* can't handle these yet */
	    TIFFClose(image->info.tiff.tif);
	    return -1;
    }

    TIFFGetField(image->info.tiff.tif, TIFFTAG_IMAGEWIDTH, &w);
    TIFFGetField(image->info.tiff.tif, TIFFTAG_IMAGELENGTH, &h);
    TIFFGetFieldDefaulted(image->info.tiff.tif, TIFFTAG_BITSPERSAMPLE, &bpc);
    TIFFGetFieldDefaulted(image->info.tiff.tif, TIFFTAG_SAMPLESPERPIXEL, &components);
    TIFFGetFieldDefaulted(image->info.tiff.tif, TIFFTAG_COMPRESSION, &compression);

    image->width	= w;
    image->height	= h;
    image->components	= components;

    /* fetch the resolution values if found in the file */
    if (TIFFGetField(image->info.tiff.tif, TIFFTAG_XRESOLUTION, &res_x) &&
	TIFFGetField(image->info.tiff.tif, TIFFTAG_YRESOLUTION, &res_y) &&
	TIFFGetField(image->info.tiff.tif, TIFFTAG_RESOLUTIONUNIT, &unit) &&
	res_x > 0 && res_y > 0) {

	if (unit == RESUNIT_INCH) {
	    image->dpi_x = res_x;
	    image->dpi_y = res_y;

	} else if (unit == RESUNIT_CENTIMETER) {
	    image->dpi_x = res_x * (float) 2.54;
	    image->dpi_y = res_y * (float) 2.54;

	} else if (unit == RESUNIT_NONE) {
	    image->dpi_x = -res_x;
	    image->dpi_y = -res_y;
	}
    }

    /* special handling for CCITT-compressed TIFF images */
#define PDF_MAX_PARAMSTRING	64
    switch ((int) compression) {
	case COMPRESSION_CCITTFAX4:
	    if (bpc != 1) {
		TIFFClose(image->info.tiff.tif);
		return -1;
	    }
	    if ((int) TIFFNumberOfStrips(image->info.tiff.tif) != 1) {
		image->bpc = 8;
		break;
	    }

	    image->params = (char *) p->malloc(p, PDF_MAX_PARAMSTRING,
		"pdf_open_TIFF_data");
	    image->params[0] = '\0';

	    strcpy(image->params, "/K -1");

	    if (TIFFGetField(image->info.tiff.tif,
	    	TIFFTAG_PHOTOMETRIC, &photometric) &&
		photometric == PHOTOMETRIC_MINISBLACK)
		strcat(image->params, "/BlackIs1 true");

	    image->compression = ccitt;
	    image->bpc = 1;
	    break;

	case COMPRESSION_CCITTFAX3:
	    if (bpc != 1) {
		TIFFClose(image->info.tiff.tif);
		return -1;
	    }
	    if ((int) TIFFNumberOfStrips(image->info.tiff.tif) != 1) {
		image->bpc = 8;
		break;
	    }

	    if (TIFFGetField(image->info.tiff.tif,
	    	TIFFTAG_GROUP3OPTIONS, &group3opts)) {
		/* /K = 0 (= G3,1D) is default */
		if (group3opts & GROUP3OPT_2DENCODING) {
		    if (image->params == NULL) {
			image->params =
			    (char *) p->malloc(p, PDF_MAX_PARAMSTRING,
			    "pdf_open_TIFF_data");
			image->params[0] = '\0';
		    }
		    strcat(image->params, "/K 1");
		}

		if (group3opts & GROUP3OPT_FILLBITS) {
		    if (image->params == NULL) {
			image->params = 
			    (char *) p->malloc(p, PDF_MAX_PARAMSTRING,
			    "pdf_open_TIFF_data");
			image->params[0] = '\0';
		    }
		    strcat(image->params, "/EncodedByteAlign true");
		}
	    }

	    if (TIFFGetField(image->info.tiff.tif,
	    	TIFFTAG_PHOTOMETRIC, &photometric) &&
		(photometric == PHOTOMETRIC_MINISBLACK)) {
		if (image->params == NULL) {
		    image->params = 
			(char *) p->malloc(p, PDF_MAX_PARAMSTRING,
		    	"pdf_open_TIFF_data");
		    image->params[0] = '\0';
		}
		strcat(image->params, "/BlackIs1 true");
	    }

	    image->compression = ccitt;
	    image->bpc = 1;
	    break;

	default:
	    image->bpc = 8;
    }

    if (stringparam && *stringparam) {
	if (!strcmp(stringparam, "mask")) {
	    if (image->components != 1 || image->bpc != 1) {
		TIFFClose(image->info.tiff.tif);
		pdf_error(p, PDF_ValueError, "Cannot handle TIFF image mask");
	    }
	    image->colorspace = ImageMask;

	} else if (!strcmp(stringparam, "masked")) {
	    mask = intparam;
	    if (mask >= 0 &&
		(mask >= p->images_capacity || !p->images[mask].in_use ||
		p->images[mask].colorspace != ImageMask)) {
		    TIFFClose(image->info.tiff.tif);
		    pdf_error(p, PDF_ValueError,
		    "Bad image mask (no %d) for image '%s'", mask, filename);
	    }
	} else if (!strcmp(stringparam, "invert")) {
	    image->invert = pdf_true;
	} else {
	    TIFFClose(image->info.tiff.tif);
	    pdf_error(p, PDF_ValueError,
	    	"Unknown parameter %s in pdf_open_TIFF", stringparam);
	}
    }

    image->mask = mask;

    switch (image->components) {
	case 1:
	    /* if it is a palette image we have to adapt these accordingly */
	    if (TIFFGetField(image->info.tiff.tif, TIFFTAG_COLORMAP, &rmap, &gmap, &bmap)) {
		image->colorspace	= DeviceRGB;
		image->components	= 3;
	    } else
		/* plain old grayscale image */
		image->colorspace	= DeviceGray;
	    break;

	case 3:
	case 4:	/* we rejected SEPARATED already so it must be RGB with alpha */
	    image->components = 3;
	    image->colorspace = DeviceRGB;
	    break;

	default:
	    TIFFClose(image->info.tiff.tif);
	    return -1;
    }

    image->src.private_data	= (void *) image;

    image->src.init		= pdf_data_source_TIFF_init;
    image->src.fill		= pdf_data_source_TIFF_fill;
    image->src.terminate	= pdf_data_source_TIFF_terminate;

    image->info.tiff.raster = (uint32 *) NULL;

    if (image->compression == none) {
	npixels = (size_t) (w * h);
	image->info.tiff.raster = (uint32 *) p->malloc(p,
	    (size_t) (npixels * sizeof (uint32)), "pdf_open_TIFF");

	if (!TIFFReadRGBAImage(image->info.tiff.tif,
		(uint32) w, (uint32) h, image->info.tiff.raster, 1)) {
	    p->free(p, (void *) image->info.tiff.raster);
	    TIFFClose(image->info.tiff.tif);
	    return -1;
	}
    }

    image->in_use = pdf_true;			/* mark slot as used */
    image->filename = pdf_strdup(p, filename);

    pdf_put_image(p, imageslot);

    if (image->info.tiff.raster)
	p->free(p, (void *) image->info.tiff.raster);

    TIFFClose(image->info.tiff.tif);

    return imageslot;
}

#endif	/* HAVE_LIBTIFF */
